##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te
from pydantic import Field

from atproto.xrpc_client.models import base


class Main(base.RecordModelBase):

    """Record model for :obj:`app.bsky.graph.block`."""

    created_at: str = Field(alias='createdAt')  #: Created at.
    subject: str  #: Subject.

    py_type: te.Literal['app.bsky.graph.block'] = Field(default='app.bsky.graph.block', alias='$type', frozen=True)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te

from atproto.xrpc_client.models import base


class Data(base.DataModelBase):

    """Input data model for :obj:`app.bsky.graph.muteActorList`."""

    list: str  #: List.


class DataDict(te.TypedDict):
    list: str  #: List.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
from atproto.xrpc_client.models import base


class Response(base.ResponseModelBase):

    """Output data model for :obj:`com.atproto.server.describeServer`."""

    available_user_domains: t.List[str] = Field(alias='availableUserDomains')  #: Available user domains.
    invite_code_required: t.Optional[bool] = Field(default=None, alias='inviteCodeRequired')  #: Invite code required.
    links: t.Optional['models.ComAtprotoServerDescribeServer.Links'] = None  #: Links.


class Links(base.ModelBase):

    """Definition model for :obj:`com.atproto.server.describeServer`."""

    privacy_policy: t.Optional[str] = Field(default=None, alias='privacyPolicy')  #: Privacy policy.
    terms_of_service: t.Optional[str] = Field(default=None, alias='termsOfService')  #: Terms of service.

    py_type: te.Literal['com.atproto.server.describeServer#links'] = Field(
        default='com.atproto.server.describeServer#links', alias='$type', frozen=True
    )



from typing import List

import libipld

from atproto.exceptions import DAGCBORDecodingError


class _BytesReadCounter:
    _num_bytes_read = 0

    def __call__(self, _, num_bytes_read: int) -> None:
        self._num_bytes_read += num_bytes_read

    def __int__(self) -> int:
        return self._num_bytes_read


def decode_dag(data: bytes) -> dict:
    """Decodes and returns a single data item from the given data, with the DAG-CBOR codec.

    Args:
        data: The stream of bytes or bytes with DAG-CBOR data.

    Returns:
        :obj:`dict`: Decoded DAG-CBOR.
    """

    try:
        return libipld.decode_dag_cbor(data)
    except Exception as e:  # noqa: BLE001
        raise DAGCBORDecodingError from e


def decode_dag_multi(data: bytes) -> List[dict]:
    """Decodes and returns many data items from the given data, with the DAG-CBOR codec.

    Args:
        data: The stream of bytes or bytes with DAG-CBOR data.

    Returns:
        :obj:`list` of :obj:`dict`: Decoded DAG-CBOR.
    """
    return libipld.decode_dag_cbor_multi(data)



from atproto.xrpc_client.client.base import ClientBase
from atproto.xrpc_client.namespaces import sync_ns

# TODO(MarshalX): This file should be autogenerated!


class ClientRaw(ClientBase):
    """Group all root namespaces"""

    com: 'sync_ns.ComNamespace'
    app: 'sync_ns.AppNamespace'

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        self.com = sync_ns.ComNamespace(self)
        self.app = sync_ns.AppNamespace(self)



import typing as t
from dataclasses import dataclass

import jwt as _jwt


@dataclass
class JwtPayload:
    exp: int  # expired at
    iat: int  # created at
    scope: str
    sub: str  # DID
    aud: t.Optional[str] = None  # DID
    jti: t.Optional[str] = None  # in refresh token only


def get_jwt_payload(token: str) -> JwtPayload:
    plain_payload = _jwt.decode(token, options={'verify_signature': False})
    return JwtPayload(**plain_payload)



from .car import CAR
from .cid import CID, CIDType
from .firehose import models as firehose_models
from .nsid import NSID
from .uri import AtUri
from .xrpc_client import models
from .xrpc_client.client.async_client import AsyncClient
from .xrpc_client.client.client import Client

__version__ = '0.0.33'  # placeholder. Dynamic version from Git Tag
__all__ = [
    '__version__',
    'AsyncClient',
    'Client',
    'models',
    'firehose_models',
    'NSID',
    'CAR',
    'CID',
    'CIDType',
    'AtUri',
]



import typing as t
from dataclasses import dataclass
from datetime import timedelta

import typing_extensions as te

from atproto.xrpc_client.client.auth import get_jwt_payload

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
    from atproto.xrpc_client.client.auth import JwtPayload


@dataclass
class SessionString:
    access_jwt: str
    refresh_jwt: str
    did: str
    handle: str

    _SESSION_STRING_SEPARATOR: te.Final[te.Literal[':::']] = ':::'

    def encode(self) -> str:
        payload = [
            self.handle,
            self.did,
            self.access_jwt,
            self.refresh_jwt,
        ]
        return self._SESSION_STRING_SEPARATOR.join(payload)

    @classmethod
    def decode(cls, string: str) -> 'SessionString':
        handle, did, access_jwt, refresh_jwt = string.split(cls._SESSION_STRING_SEPARATOR)
        return cls(access_jwt, refresh_jwt, did, handle)


SessionResponse: te.TypeAlias = t.Union[
    'models.ComAtprotoServerCreateSession.Response',
    'models.ComAtprotoServerRefreshSession.Response',
    'SessionString',
]


class SessionMethodsMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._access_jwt: t.Optional[str] = None
        self._access_jwt_payload: t.Optional['JwtPayload'] = None

        self._refresh_jwt: t.Optional[str] = None
        self._refresh_jwt_payload: t.Optional['JwtPayload'] = None

        self._session: t.Optional[SessionString] = None

    def _should_refresh_session(self) -> bool:
        expired_at = self.get_time_from_timestamp(self._access_jwt_payload.exp)
        expired_at = expired_at - timedelta(minutes=15)  # let's update the token a bit later than required

        return self.get_current_time() > expired_at

    def _set_session(self, session: SessionResponse) -> None:
        self._access_jwt = session.access_jwt
        self._access_jwt_payload = get_jwt_payload(session.access_jwt)

        self._refresh_jwt = session.refresh_jwt
        self._refresh_jwt_payload = get_jwt_payload(session.refresh_jwt)

        self._session = SessionString(
            access_jwt=session.access_jwt,
            refresh_jwt=session.refresh_jwt,
            did=session.did,
            handle=session.handle,
        )

        self._set_auth_headers(session.access_jwt)

    @staticmethod
    def _get_auth_headers(token: str) -> t.Dict[str, str]:
        return {'Authorization': f'Bearer {token}'}

    def _set_auth_headers(self, token: str) -> None:
        self.request.set_additional_headers(self._get_auth_headers(token))

    def _import_session_string(self, string: str) -> SessionString:
        session = SessionString.decode(string)
        self._set_session(session)
        return session

    def export_session_string(self) -> str:
        """Export session string.

        Note:
            This method is useful for storing the session and reusing it later.

        Warning:
            You should use it if you create the client instance often.
            Because of server rate limits for `createSession`.
            Rate limited by handle.
            30/5 min, 300/day.

        Example:
            >>> from atproto import Client
            >>> # the first time login with login and password
            >>> client = Client()
            >>> client.login('login', 'password')
            >>> session_string = client.export_session_string()
            >>> # store session_string somewhere.
            >>> # for example, in env and next time use it for login
            >>> client2 = Client()
            >>> client2.login(session_string=session_string)

        Returns:
            :obj:`str`: Session string.
        """
        return self._session.encode()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################

import typing as t
from asyncio import Lock

from atproto.utils import TextBuilder
from atproto.xrpc_client import models
from atproto.xrpc_client.client.async_raw import AsyncClientRaw
from atproto.xrpc_client.client.methods_mixin import SessionMethodsMixin, TimeMethodsMixin
from atproto.xrpc_client.models import ids
from atproto.xrpc_client.models.languages import DEFAULT_LANGUAGE_CODE1

if t.TYPE_CHECKING:
    from atproto.xrpc_client.client.base import InvokeType
    from atproto.xrpc_client.request import Response


class AsyncClient(SessionMethodsMixin, TimeMethodsMixin, AsyncClientRaw):
    """High-level client for XRPC of ATProto."""

    def __init__(self, base_url: t.Optional[str] = None, *args, **kwargs) -> None:
        super().__init__(base_url, *args, **kwargs)

        self._refresh_lock = Lock()

        self.me: t.Optional[models.AppBskyActorDefs.ProfileViewDetailed] = None

    async def _invoke(self, invoke_type: 'InvokeType', **kwargs) -> 'Response':
        session_refreshing = kwargs.pop('session_refreshing', False)
        if session_refreshing:
            return await super()._invoke(invoke_type, **kwargs)

        async with self._refresh_lock:
            if self._access_jwt and self._should_refresh_session():
                await self._refresh_and_set_session()

        return await super()._invoke(invoke_type, **kwargs)

    async def _get_and_set_session(self, login: str, password: str) -> models.ComAtprotoServerCreateSession.Response:
        session = await self.com.atproto.server.create_session(
            models.ComAtprotoServerCreateSession.Data(identifier=login, password=password)
        )
        self._set_session(session)

        return session

    async def _refresh_and_set_session(self) -> models.ComAtprotoServerRefreshSession.Response:
        refresh_session = await self.com.atproto.server.refresh_session(
            headers=self._get_auth_headers(self._refresh_jwt), session_refreshing=True
        )
        self._set_session(refresh_session)

        return refresh_session

    async def login(
        self, login: t.Optional[str] = None, password: t.Optional[str] = None, session_string: t.Optional[str] = None
    ) -> models.AppBskyActorDefs.ProfileViewDetailed:
        """Authorize a client and get profile info.

        Args:
            login: Handle/username of the account.
            password: Main or app-specific password of the account.
            session_string: Session string (use :py:attr:`~export_session_string` to obtain it).

        Note:
            Either `session_string` or `login` and `password` should be provided.

        Returns:
            :obj:`models.AppBskyActorDefs.ProfileViewDetailed`: Profile information.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """
        if session_string:
            session = self._import_session_string(session_string)
        elif login and password:
            session = await self._get_and_set_session(login, password)
        else:
            raise ValueError('Either session_string or login and password should be provided.')

        self.me = await self.app.bsky.actor.get_profile(models.AppBskyActorGetProfile.Params(actor=session.handle))
        return self.me

    async def send_post(
        self,
        text: t.Union[str, TextBuilder],
        profile_identify: t.Optional[str] = None,
        reply_to: t.Optional[t.Union[models.AppBskyFeedPost.ReplyRef, models.AppBskyFeedDefs.ReplyRef]] = None,
        embed: t.Optional[
            t.Union[
                'models.AppBskyEmbedImages.Main',
                'models.AppBskyEmbedExternal.Main',
                'models.AppBskyEmbedRecord.Main',
                'models.AppBskyEmbedRecordWithMedia.Main',
            ]
        ] = None,
        langs: t.Optional[t.List[str]] = None,
        facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = None,
    ) -> models.ComAtprotoRepoCreateRecord.Response:
        """Send post.

        Note:
            If `profile_identify` is not provided will be sent to the current profile.

            The async default language is ``en``.
            Available languages are async defined in :py:mod:`atproto.xrpc_client.models.languages`.

        Args:
            text: Text of the post.
            profile_identify: Handle or DID. Where to send post.
            reply_to: Root and parent of the post to reply to.
            embed: Embed models that should be attached to the post.
            langs: List of used languages in the post.
            facets: List of facets (rich text items).

        Returns:
            :obj:`models.ComAtprotoRepoCreateRecord.Response`: Reference to the created post record.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """
        if isinstance(text, TextBuilder):
            facets = text.build_facets()
            text = text.build_text()

        repo = self.me.did
        if profile_identify:
            repo = profile_identify

        if not langs:
            langs = [DEFAULT_LANGUAGE_CODE1]

        return await self.com.atproto.repo.create_record(
            models.ComAtprotoRepoCreateRecord.Data(
                repo=repo,
                collection=ids.AppBskyFeedPost,
                record=models.AppBskyFeedPost.Main(
                    created_at=self.get_current_time_iso(),
                    text=text,
                    reply=reply_to,
                    embed=embed,
                    langs=langs,
                    facets=facets,
                ),
            )
        )

    async def send_image(
        self,
        text: t.Union[str, TextBuilder],
        image: bytes,
        image_alt: str,
        profile_identify: t.Optional[str] = None,
        reply_to: t.Optional[t.Union[models.AppBskyFeedPost.ReplyRef, models.AppBskyFeedDefs.ReplyRef]] = None,
        langs: t.Optional[t.List[str]] = None,
        facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = None,
    ) -> models.ComAtprotoRepoCreateRecord.Response:
        """Send post with attached image.

        Note:
            If `profile_identify` is not provided will be sent to the current profile.

        Args:
            text: Text of the post.
            image: Binary image to attach.
            image_alt: Text version of the image
            profile_identify: Handle or DID. Where to send post.
            reply_to: Root and parent of the post to reply to.
            langs: List of used languages in the post.
            facets: List of facets (rich text items).

        Returns:
            :obj:`models.ComAtprotoRepoCreateRecord.Response`: Reference to the created post record.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """

        upload = await self.com.atproto.repo.upload_blob(image)
        images = [models.AppBskyEmbedImages.Image(alt=image_alt, image=upload.blob)]
        return await self.send_post(
            text,
            profile_identify=profile_identify,
            reply_to=reply_to,
            embed=models.AppBskyEmbedImages.Main(images=images),
            langs=langs,
            facets=facets,
        )

    async def like(self, subject: models.ComAtprotoRepoStrongRef.Main) -> models.ComAtprotoRepoCreateRecord.Response:
        """Like the post.

        Args:
            subject: Reference to the post that should be liked.

        Returns:
            :obj:`models.ComAtprotoRepoCreateRecord.Response`: Reference to the created like record.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """

        return await self.com.atproto.repo.create_record(
            models.ComAtprotoRepoCreateRecord.Data(
                repo=self.me.did,
                collection=ids.AppBskyFeedLike,
                record=models.AppBskyFeedLike.Main(created_at=self.get_current_time_iso(), subject=subject),
            )
        )

    async def unlike(self, record_key: str, profile_identify: t.Optional[str] = None) -> bool:
        """Unlike the post.

        Args:
            record_key: ID (slog) of the post.
            profile_identify: Handler or DID. Who did the like.

        Returns:
            :obj:`bool`: Success status.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """

        repo = self.me.did
        if profile_identify:
            repo = profile_identify

        return await self.com.atproto.repo.delete_record(
            models.ComAtprotoRepoDeleteRecord.Data(
                collection=ids.AppBskyFeedLike,
                repo=repo,
                rkey=record_key,
            )
        )

    async def repost(
        self,
        subject: models.ComAtprotoRepoStrongRef.Main,
        profile_identify: t.Optional[str] = None,
    ) -> models.ComAtprotoRepoCreateRecord.Response:
        """Repost post.

        Note:
            If `profile_identify` is not provided will be sent to the current profile.

        Args:
            subject: Reference to the post that should be reposted.
            profile_identify: Handle or DID. Where to make repost.

        Returns:
            :obj:`models.ComAtprotoRepoCreateRecord.Response`: Reference to the reposted post record.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """

        repo = self.me.did
        if profile_identify:
            repo = profile_identify

        return await self.com.atproto.repo.create_record(
            models.ComAtprotoRepoCreateRecord.Data(
                repo=repo,
                collection=ids.AppBskyFeedRepost,
                record=models.AppBskyFeedRepost.Main(
                    created_at=self.get_current_time_iso(),
                    subject=subject,
                ),
            )
        )

    async def delete_post(self, post_rkey: str, profile_identify: t.Optional[str] = None) -> bool:
        """Delete post.

        Args:
            post_rkey: ID (slug) of the post.
            profile_identify: Handler or DID. Who created the post.

        Returns:
            :obj:`bool`: Success status.

        Raises:
            :class:`atproto.exceptions.AtProtocolError`: Base exception.
        """

        repo = self.me.did
        if profile_identify:
            repo = profile_identify

        return await self.com.atproto.repo.delete_record(
            models.ComAtprotoRepoDeleteRecord.Data(
                collection=ids.AppBskyFeedPost,
                repo=repo,
                rkey=post_rkey,
            )
        )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
from atproto.xrpc_client.models import base


class Params(base.ParamsModelBase):

    """Parameters model for :obj:`app.bsky.feed.getPostThread`."""

    uri: str  #: Uri.
    depth: t.Optional[int] = Field(default=6, ge=0, le=1000)  #: Depth.
    parent_height: t.Optional[int] = Field(default=80, alias='parentHeight', ge=0, le=1000)  #: Parent height.


class ParamsDict(te.TypedDict):
    uri: str  #: Uri.
    depth: te.NotRequired[t.Optional[int]]  #: Depth.
    parent_height: te.NotRequired[t.Optional[int]]  #: Parent height.


class Response(base.ResponseModelBase):

    """Output data model for :obj:`app.bsky.feed.getPostThread`."""

    thread: te.Annotated[
        t.Union[
            'models.AppBskyFeedDefs.ThreadViewPost',
            'models.AppBskyFeedDefs.NotFoundPost',
            'models.AppBskyFeedDefs.BlockedPost',
        ],
        Field(discriminator='py_type'),
    ]  #: Thread.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client.models.unknown_type import UnknownInputType
from atproto.xrpc_client.models import base


class Data(base.DataModelBase):

    """Input data model for :obj:`com.atproto.repo.putRecord`."""

    collection: str  #: The NSID of the record collection.
    record: 'UnknownInputType'  #: The record to write.
    repo: str  #: The handle or DID of the repo.
    rkey: str = Field(max_length=15)  #: The key of the record.
    swap_commit: t.Optional[str] = Field(
        default=None, alias='swapCommit'
    )  #: Compare and swap with the previous commit by CID.
    swap_record: t.Optional[str] = Field(
        default=None, alias='swapRecord'
    )  #: Compare and swap with the previous record by CID.
    validate_: t.Optional[bool] = Field(default=True, alias='validate')  #: Flag for validating the record.


class DataDict(te.TypedDict):
    collection: str  #: The NSID of the record collection.
    record: 'UnknownInputType'  #: The record to write.
    repo: str  #: The handle or DID of the repo.
    rkey: str  #: The key of the record.
    swap_commit: te.NotRequired[t.Optional[str]]  #: Compare and swap with the previous commit by CID.
    swap_record: te.NotRequired[t.Optional[str]]  #: Compare and swap with the previous record by CID.
    validate: te.NotRequired[t.Optional[bool]]  #: Flag for validating the record.


class Response(base.ResponseModelBase):

    """Output data model for :obj:`com.atproto.repo.putRecord`."""

    cid: str  #: Cid.
    uri: str  #: Uri.



import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
    from atproto.xrpc_client.models import dot_dict

UnknownRecordType: te.TypeAlias = t.Union[
    'models.AppBskyActorProfile.Main',
    'models.AppBskyFeedGenerator.Main',
    'models.AppBskyFeedLike.Main',
    'models.AppBskyFeedPost.Main',
    'models.AppBskyFeedRepost.Main',
    'models.AppBskyFeedThreadgate.Main',
    'models.AppBskyGraphBlock.Main',
    'models.AppBskyGraphFollow.Main',
    'models.AppBskyGraphList.Main',
    'models.AppBskyGraphListblock.Main',
    'models.AppBskyGraphListitem.Main',
]
UnknownRecordTypePydantic = te.Annotated[
    t.Union[
        'models.AppBskyActorProfile.Main',
        'models.AppBskyFeedGenerator.Main',
        'models.AppBskyFeedLike.Main',
        'models.AppBskyFeedPost.Main',
        'models.AppBskyFeedRepost.Main',
        'models.AppBskyFeedThreadgate.Main',
        'models.AppBskyGraphBlock.Main',
        'models.AppBskyGraphFollow.Main',
        'models.AppBskyGraphList.Main',
        'models.AppBskyGraphListblock.Main',
        'models.AppBskyGraphListitem.Main',
    ],
    Field(discriminator='py_type'),
]
UnknownType: te.TypeAlias = t.Union[UnknownRecordTypePydantic, 'dot_dict.DotDictType']
UnknownInputType: te.TypeAlias = t.Union[UnknownType, t.Dict[str, t.Any]]



from pydantic import BaseModel, ConfigDict

from atproto.exceptions import ModelFieldNotFoundError


class AtProtocolBase:
    ...


class ModelBase(BaseModel, AtProtocolBase):
    """Base class for all data classes.

    Provides square brackets [] notation to get attributes like in a dictionary.
    """

    model_config = ConfigDict(extra='forbid', populate_by_name=True, strict=True)

    def __getitem__(self, item: str):
        if hasattr(self, item):
            return getattr(self, item)

        raise ModelFieldNotFoundError(f"Can't find field '{item}' in the object of type {type(self)}.")


class ParamsModelBase(ModelBase):
    pass


class DataModelBase(ModelBase):
    pass


class ResponseModelBase(ModelBase):
    pass


class UnknownDict(AtProtocolBase):
    pass


class UnknownRecord(ModelBase):
    pass


class RecordModelBase(UnknownRecord):
    pass



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
    from atproto.xrpc_client.models.blob_ref import BlobRef
from atproto.xrpc_client.models import base


class Main(base.RecordModelBase):

    """Record model for :obj:`app.bsky.feed.generator`."""

    created_at: str = Field(alias='createdAt')  #: Created at.
    did: str  #: Did.
    display_name: str = Field(alias='displayName', max_length=240)  #: Display name.
    avatar: t.Optional['BlobRef'] = None  #: Avatar.
    description: t.Optional[str] = Field(default=None, max_length=3000)  #: Description.
    description_facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = Field(
        default=None, alias='descriptionFacets'
    )  #: Description facets.
    labels: t.Optional[
        te.Annotated[t.Union['models.ComAtprotoLabelDefs.SelfLabels'], Field(default=None, discriminator='py_type')]
    ] = None  #: Labels.

    py_type: te.Literal['app.bsky.feed.generator'] = Field(
        default='app.bsky.feed.generator', alias='$type', frozen=True
    )



import typing as t
from dataclasses import dataclass
from enum import Enum

Number = t.Union[int, float, complex]


class LexDefinitionType(Enum):
    RECORD = 'record'

    QUERY = 'query'
    PROCEDURE = 'procedure'
    SUBSCRIPTION = 'subscription'

    PARAMS = 'params'
    TOKEN = 'token'  # noqa: S105
    OBJECT = 'object'

    BLOB = 'blob'
    ARRAY = 'array'

    STRING = 'string'  # TODO(MarshalX): definitions could be primitives?


class LexPrimitiveType(Enum):
    BOOLEAN = 'boolean'
    NUMBER = 'number'
    INTEGER = 'integer'
    STRING = 'string'
    REF = 'ref'
    UNION = 'union'
    UNKNOWN = 'unknown'
    CID_LINK = 'cid-link'
    BYTES = 'bytes'


@dataclass
class LexDefinition:  # original name LexUserType
    type: LexDefinitionType
    description: t.Optional[str]


@dataclass
class LexPrimitive:
    type: LexPrimitiveType
    description: t.Optional[str]


@dataclass
class LexUnknown(LexPrimitive):
    type = LexPrimitiveType.UNKNOWN


@dataclass
class LexCidLink(LexPrimitive):
    type = LexPrimitiveType.CID_LINK


@dataclass
class LexBytes(LexPrimitive):
    type = LexPrimitiveType.BYTES
    maxLength: t.Optional[Number]
    minLength: t.Optional[Number]


@dataclass
class LexBoolean(LexPrimitive):
    type = LexPrimitiveType.BOOLEAN
    default: t.Optional[bool]
    const: t.Optional[bool]


@dataclass
class LexNumber(LexPrimitive):
    type = LexPrimitiveType.NUMBER
    default: t.Optional[Number]
    minimum: t.Optional[Number]
    maximum: t.Optional[Number]
    enum: t.Optional[t.List[Number]]
    const: t.Optional[Number]


@dataclass
class LexInteger(LexPrimitive):
    type = LexPrimitiveType.INTEGER
    default: t.Optional[int]
    minimum: t.Optional[int]
    maximum: t.Optional[int]
    enum: t.Optional[t.List[int]]
    const: t.Optional[int]


@dataclass
class LexString(LexPrimitive):
    type = LexPrimitiveType.STRING
    format: t.Optional[str]
    default: t.Optional[str]
    minLength: t.Optional[int]
    maxLength: t.Optional[int]
    minGraphemes: t.Optional[int]
    maxGraphemes: t.Optional[int]
    enum: t.Optional[t.List[str]]
    const: t.Optional[str]
    knownValues: t.Optional[t.List[str]]


@dataclass
class LexBlob(LexDefinition):
    type = LexDefinitionType.BLOB
    accept: t.Optional[t.List[str]]
    maxSize: t.Optional[Number]


@dataclass
class LexRef(LexPrimitive):
    type = LexPrimitiveType.REF
    ref: str


@dataclass
class LexRefUnion(LexPrimitive):
    type = LexPrimitiveType.UNION
    refs: t.List[str]
    closed: t.Optional[bool]


LexRefVariant = t.Union[LexRef, LexRefUnion]


@dataclass
class LexArray(LexDefinition):
    type = LexDefinitionType.ARRAY
    description: t.Optional[str]
    items: t.Union[LexPrimitive, LexBlob, LexRefVariant]
    minLength: t.Optional[int]
    maxLength: t.Optional[int]


@dataclass
class LexiconDoc:
    lexicon: int
    id: str  # an NSID
    revision: t.Optional[int]
    description: t.Optional[str]
    defs: t.Dict[str, LexDefinition]


@dataclass
class LexToken(LexDefinition):
    type = LexDefinitionType.TOKEN


@dataclass
class LexObject(LexDefinition):
    type = LexDefinitionType.OBJECT
    required: t.Optional[t.List[str]]
    nullable: t.Optional[t.List[str]]
    properties: t.Dict[str, t.Union[LexRefVariant, LexArray, LexPrimitive, LexBlob]]


@dataclass
class LexRecord(LexDefinition):
    type = LexDefinitionType.RECORD
    key: t.Optional[str]
    record: LexObject


@dataclass
class LexXrpcParameters(LexDefinition):
    type = LexDefinitionType.PARAMS
    description: t.Optional[str]
    required: t.Optional[t.List[str]]
    properties: t.Dict[str, t.Union[LexArray, LexPrimitive]]


@dataclass
class LexXrpcSubscriptionMessage:
    description: t.Optional[str]
    schema: t.Optional[t.Union[LexObject, LexRefVariant]]


@dataclass
class LexXrpcBody:
    description: t.Optional[str]
    encoding: t.Union[str, t.List[str]]
    schema: t.Optional[t.Union[LexObject, LexRefVariant]]


@dataclass
class LexXrpcError:
    name: str
    description: t.Optional[str]


@dataclass
class LexSubscription(LexDefinition):
    type = LexDefinitionType.SUBSCRIPTION
    parameters: t.Optional[LexXrpcParameters]
    message: t.Optional[LexXrpcSubscriptionMessage]
    infos: t.Optional[t.List[LexXrpcError]]
    errors: t.Optional[t.List[LexXrpcError]]


@dataclass
class LexXrpcQuery(LexDefinition):
    type = LexDefinitionType.QUERY
    parameters: t.Optional[LexXrpcParameters]
    output: t.Optional[LexXrpcBody]
    errors: t.Optional[t.List[LexXrpcError]]


@dataclass
class LexXrpcProcedure(LexDefinition):
    type = LexDefinitionType.PROCEDURE
    parameters: t.Optional[LexXrpcParameters]
    input: t.Optional[LexXrpcBody]
    output: t.Optional[LexXrpcBody]
    errors: t.Optional[t.List[LexXrpcError]]



import typing as t

if t.TYPE_CHECKING:
    from atproto.xrpc_client.request import Response


class AtProtocolError(Exception):
    """Base exception"""


class LexiconParsingError(AtProtocolError):
    ...


class UnknownPrimitiveTypeError(LexiconParsingError):
    ...


class UnknownDefinitionTypeError(LexiconParsingError):
    ...


class InvalidNsidError(AtProtocolError):
    ...


class ModelError(AtProtocolError):
    ...


class ModelFieldNotFoundError(ModelError):
    ...


class RequestErrorBase(AtProtocolError):
    def __init__(self, response: t.Optional['Response'] = None) -> None:
        self.response: t.Optional['Response'] = response


class NetworkError(RequestErrorBase):
    ...


class InvokeTimeoutError(NetworkError):
    ...


class UnauthorizedError(RequestErrorBase):
    ...


class RequestException(RequestErrorBase):
    ...


class BadRequestError(RequestErrorBase):
    ...


class InvalidAtUriError(AtProtocolError):
    ...


class FirehoseError(AtProtocolError):
    ...


class FirehoseDecodingError(FirehoseError):
    ...


class DAGCBORDecodingError(AtProtocolError):
    ...


class InvalidCARFile(AtProtocolError):
    ...



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client.models.unknown_type import UnknownType
from atproto.xrpc_client.models import base


class Params(base.ParamsModelBase):

    """Parameters model for :obj:`com.atproto.repo.describeRepo`."""

    repo: str  #: The handle or DID of the repo.


class ParamsDict(te.TypedDict):
    repo: str  #: The handle or DID of the repo.


class Response(base.ResponseModelBase):

    """Output data model for :obj:`com.atproto.repo.describeRepo`."""

    collections: t.List[str]  #: Collections.
    did: str  #: Did.
    did_doc: 'UnknownType' = Field(alias='didDoc')  #: Did doc.
    handle: str  #: Handle.
    handle_is_correct: bool = Field(alias='handleIsCorrect')  #: Handle is correct.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
from atproto.xrpc_client.models import base


class Main(base.RecordModelBase):

    """Record model for :obj:`app.bsky.feed.like`."""

    created_at: str = Field(alias='createdAt')  #: Created at.
    subject: 'models.ComAtprotoRepoStrongRef.Main'  #: Subject.

    py_type: te.Literal['app.bsky.feed.like'] = Field(default='app.bsky.feed.like', alias='$type', frozen=True)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
from atproto.xrpc_client.models import base


class MentionRule(base.ModelBase):

    """Definition model for :obj:`app.bsky.feed.threadgate`. Allow replies from actors mentioned in your post."""

    py_type: te.Literal['app.bsky.feed.threadgate#mentionRule'] = Field(
        default='app.bsky.feed.threadgate#mentionRule', alias='$type', frozen=True
    )


class FollowingRule(base.ModelBase):

    """Definition model for :obj:`app.bsky.feed.threadgate`. Allow replies from actors you follow."""

    py_type: te.Literal['app.bsky.feed.threadgate#followingRule'] = Field(
        default='app.bsky.feed.threadgate#followingRule', alias='$type', frozen=True
    )


class ListRule(base.ModelBase):

    """Definition model for :obj:`app.bsky.feed.threadgate`. Allow replies from actors on a list."""

    list: str  #: List.

    py_type: te.Literal['app.bsky.feed.threadgate#listRule'] = Field(
        default='app.bsky.feed.threadgate#listRule', alias='$type', frozen=True
    )


class Main(base.RecordModelBase):

    """Record model for :obj:`app.bsky.feed.threadgate`."""

    created_at: str = Field(alias='createdAt')  #: Created at.
    post: str  #: Post.
    allow: t.Optional[
        t.List[
            te.Annotated[
                t.Union[
                    'models.AppBskyFeedThreadgate.MentionRule',
                    'models.AppBskyFeedThreadgate.FollowingRule',
                    'models.AppBskyFeedThreadgate.ListRule',
                ],
                Field(discriminator='py_type'),
            ]
        ]
    ] = Field(default=None, max_length=5)  #: Allow.

    py_type: te.Literal['app.bsky.feed.threadgate'] = Field(
        default='app.bsky.feed.threadgate', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto.xrpc_client import models
from atproto.xrpc_client.models import base


class Params(base.ParamsModelBase):

    """Parameters model for :obj:`com.atproto.sync.listRepos`."""

    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=500, ge=1, le=1000)  #: Limit.


class ParamsDict(te.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.


class Response(base.ResponseModelBase):

    """Output data model for :obj:`com.atproto.sync.listRepos`."""

    repos: t.List['models.ComAtprotoSyncListRepos.Repo']  #: Repos.
    cursor: t.Optional[str] = None  #: Cursor.


class Repo(base.ModelBase):

    """Definition model for :obj:`com.atproto.sync.listRepos`."""

    did: str  #: Did.
    head: str  #: Head.
    rev: str  #: Rev.

    py_type: te.Literal['com.atproto.sync.listRepos#repo'] = Field(
        default='com.atproto.sync.listRepos#repo', alias='$type', frozen=True
    )
